import traceback
import ast
import os

def describe_error(e, args, kwargs):
    print("An error occurred: " + str(e) + "\n" + "\n")
    get_project_files(e)





def get_project_files(e):
    
    # stack trace
    stack_trace_list = traceback.format_exc().split("\n")

    # get all file names in order
    file_names = []
    for i, line in enumerate(stack_trace_list):
        if line.startswith("  File"):
            file_names.append(line.split(",")[0].split('"')[1])

    # standardize file names
    for i, file_path in enumerate(file_names):
        file_names[i] = file_path.replace("\\", "/")

    project_root = "/".join(file_names[1].split("/")[0:-1])


    # get all files in directory, recursively

    all_files_text = ""

    for file_path in search_directory(project_root):

        with open(file_path, 'r') as f:

            file_name = file_path.split("/")[-1]

            all_files_text += "\n\n" + file_name + "\n" + \
                "```python\n" + f.read() + "\n```"
    

        with open("text.txt", 'w') as f:
            f.write(all_files_text)

    



# recursively go through a directory and return a generator of all files
def search_directory(directory):
    for (dirpath, dirnames, filenames) in os.walk(directory):
        for filename in filenames:
            if (filename.endswith(".py")):
                yield os.path.join(dirpath, filename)
    




    # for i, line in enumerate(stack_trace_list):
    #     if line.startswith("  File"):
    #         # open file
    #         file_name = line.split(",")[0].split('"')[1]
    #         print(file_name)
    #         with open(file_name, 'r') as f:
    #             tree = ast.parse(f.read())
    #             for node in ast.walk(tree):
    #                 if isinstance(node, ast.FunctionDef):
    #                     print(node.name)
    #                     print(node.lineno)

            # get line number


    # print("An error occurred: " + str(e) + "\n" + "\n".join(stack_trace_list))




import os
import re

dependencyGraph = {}

# Get all files in the current working directory
directory = os.getcwd()
for root, dirs, files in os.walk(directory):
    for filename in files:
        f = os.path.join(root, filename)
        dependencyGraph[f] = []

# Remove the current file
current_file_path = os.path.abspath(__file__)
del dependencyGraph[current_file_path]

# Build Graph
for key, lis in dependencyGraph.items():
    with open(key, 'r') as curFile:
        lines = curFile.readlines()
        for line in lines:
            match = re.match(r'^\s*from\s+(\S+)\s+import\s+\S+', line)
            if match:
                imported_module = match.group(1)
                dependencyGraph[key].append(imported_module)
            else:
                match = re.match(r'^\s*import\s+(\S+)', line)
                if match:
                    imported_module = match.group(1)
                    dependencyGraph[key].append(imported_module)



import functools
from .describe_error import describe_error

def with_handler(func):
    @functools.wraps(func)
    def wrapper(*args, **kwargs):
        try:
            return func(*args, **kwargs)
        except Exception as e:
            describe_error(e, args, kwargs)
    return wrapper



from colorama import init as colorama_init
from colorama import Fore
from colorama import Style

colorama_init()


def colorOutput(prompt, keywords):
    for i in keywords:
        prompt = prompt.replace(i, f"{Fore.RED}{i}{Style.RESET_ALL}")
    return prompt
keywords = ["line", "syntax", "compile time", "run time", "error"]
prompt = "On line 19, there is a syntax error causing a compile time error. To resolve this, ..."






generation_color = Fore.WHITE

is_code_block = False

def color_terminal_code_blocks(text):
    # replace all instances of ``` with green color

    for char in text:
        if char == '`':
            global is_code_block
            is_code_block = not is_code_block
            if is_code_block:
                text = text.replace('`', Fore.GREEN + '`', 1)
            else:
                text = text.replace('`', generation_color + '`', 1)

    return text



import sys
import subprocess
from openai import OpenAI
import glob
import os
import platform
from .get_openai_key import get_key
from .color import generation_color, color_terminal_code_blocks


def get_and_format_environment_details():
    python_version = sys.version
    operating_system = platform.platform()
    installed_packages = subprocess.getoutput("pip list")
    formatted_details = f"Python Version: {python_version}\n" + \
                        f"Operating System: {operating_system}\n" + \
                        f"Installed Packages:\n{installed_packages}"
    return formatted_details

def get_python_files_contents(script_path):
    script_dir = os.path.dirname(script_path)
    python_files = glob.glob(os.path.join(script_dir, '**', '*.py'), recursive=True)
    files_contents = ""
    for file in python_files:
        with open(file, 'r') as f:
            files_contents += f"\nFile: {os.path.basename(file)}\n\n"
            files_contents += f.read() + "\n"
    return files_contents


def explain_error_with_gpt(error_message, script_path):
    script_dir = os.path.dirname(script_path)

    client = OpenAI(api_key=get_key(script_path))

    try:
        response = client.chat.completions.create(
            model="gpt-3.5-turbo-1106",
            messages=[
                {"role": "system", "content": "You are a python expert that gives specific code suggestions."},
                {"role": "user", "content": f"{error_message}\n\nUnderstand the following exception, along with all of the associated functions. Explain what might have caused it and suggest possible fixes. Mention line numbers and include code blocks. When using line numbers. Be concise and specific."}
            ],
            stream=True
        )

        print(generation_color, end="")
        for chunk in response:
            print(color_terminal_code_blocks( chunk.choices[0].delta.content), end="")

    except Exception as e:
        return f"Error in contacting OpenAI API: {str(e)}"



def run_script(script_path, custom_prompt=""):
    try:
        output = subprocess.check_output([sys.executable, script_path], stderr=subprocess.STDOUT)
        print(output.decode())
    except subprocess.CalledProcessError as e:
        error_output = e.output.decode()

        # color red
        print("\033[91m", end="")
        print(error_output)
        env_details = get_and_format_environment_details()
        all_files_contents = get_python_files_contents(script_path)
        combined_input = f"Environment Details:\n{env_details}\n\nPython Files Contents:\n{all_files_contents}\n\nError Output:\n{error_output}\n\n{custom_prompt}"

        explain_error_with_gpt(combined_input, script_path)
        print("\033[0m")

def main():
    if len(sys.argv) < 2:
        print("Usage: mhacks-run <script_path>")
        sys.exit(1)

    script_path = sys.argv[1]
    custom_prompt = sys.argv[2] if len(sys.argv) > 2 else ""
    run_script(script_path, custom_prompt)



import os
from dotenv import load_dotenv

def get_key(root_dir):
  # check if key is already stored in .env

  # if not, prompt user for key and store in .env
  # return key
  dotenv_path = os.path.join(root_dir, '.env')
  load_dotenv(dotenv_path)

  if os.getenv('OPENAI_API_KEY'):
    return os.getenv('OPENAI_API_KEY')
  
  key = input("Enter your OpenAI API key: ")
  with open(dotenv_path, 'a') as f:
    f.write(f"\nOPENAI_API_KEY={key}")

  return key



import setuptools


version = {}
with open("src/swibin/version.py", "r") as f:
    exec(f.read(), version)

version = version["__version__"]

setuptools.setup(
    name="swibin",
    version=version,
    author="Swibin Team",
    author_email="your.email@example.com",
    description="Smart error insights for Python",
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
    ],
    python_requires='>=3.8',
    install_requires=[
        'openai==1.3.3',
        'python-dotenv'
    ],
    entry_points={
        'console_scripts': [
            'swibin=swibin.cli:main',
        ],
    },
)
